<?php
/*
Plugin Name: Content Engage
Plugin URI: http://appbox.im/
Description: Split test your content easily.
Version: 1.0.0
Author: Andrei-Robert Rusu
Author URI: http://appbox.im/
Author Email: marius@digitalfollow.com
*/

class ContentEngageController {

  protected static $_instance;

  public static function getInstance() {
    if(self::$_instance === null)
      self::$_instance = new self();

    return self::$_instance;
  }

  public $pluginName         = 'ContentEngage';
  public $pluginSlug         = 'content_engage';
  public $pluginPrefix       = 'content_engage';
  public $pluginActionPrefix = 'content_engage';

  public $pluginFilePath;
  public $pluginURLPath;

  public $postOptionSplitTestingMargin = 'content_engage_split_testing_margin';
  public $postOptionTestMode = 'content_engage_test_mode';

  public $footerScript = '';

  /**
   * @var ContentEngageVersionControl
   */
  public $modelVersionControl;
  /**
   * @var ContentEngageBackendRequest
   */
  public $modelBackendRequest;
  /**
   * @var ContentEngageSettings
   */
  public $modelSettings;
  /**
   * @var ContentEngagePostWidget
   */
  public $modelPostWidget;
  /**
   * @var ContentEngageTitleHandler
   */
  public $modelTitleHandler;
  /**
   * @var ContentEngageImageHandler
   */
  public $modelImageHandler;
  /**
   * @var ContentEngageExcerptHandler
   */
  public $modelExcerptHandler;
  /**
   * @var ContentEngagePostTitle
   */
  public $entityPostTitle;
  /**
   * @var ContentEngagePostAssign
   */
  public $entityPostAssign;
  /**
   * @var ContentEngagePostDisplay
   */
  public $entityPostDisplay;
  /**
   * @var ContentEngagePostImage
   */
  public $entityPostImage;
  /**
   * @var ContentEngagePostImageAssign
   */
  public $entityPostImageAssign;
  /**
   * @var ContentEngagePostImageDisplay
   */
  public $entityPostImageDisplay;
  /**
   * @var ContentEngagePostExcerpt
   */
  public $entityPostExcerpt;
  /**
   * @var ContentEngagePostExcerptAssign
   */
  public $entityPostExcerptAssign;
  /**
   * @var ContentEngagePostExcerptDisplay
   */
  public $entityPostExcerptDisplay;

  public $chartStatisticsColors = array(
      array( "default" => "#8ec45a", "hover" => "#a6d37a" ),
      array( "default" => "#34a1da", "hover" => "#50b3e7" ),
      array( "default" => "#e4902e", "hover" => "#eaa049" ),
      array( "default" => "#e4322e", "hover" => "#f04a47" ),
      array( "default" => "#cb49b1", "hover" => "#e264c9" ),
      array( "default" => "#9949cb", "hover" => "#ab5bdc" ),
      array( "default" => "#49cb7a", "hover" => "#68df95" ),
      array( "default" => "#3641d6", "hover" => "#515ce6" ),
      array( "default" => "#3a9b48", "hover" => "#4eaf5c" ),
      array( "default" => "#01cbd8", "hover" => "#4ad7e0" ),
      array( "default" => "#fc4349", "hover" => "#fd5c62" ),
      array( "default" => "#047878", "hover" => "#138787" ),
      array( "default" => "#f23c55", "hover" => "#ef465d" ),
      array( "default" => "#4192d9", "hover" => "#5299d8" ),
      array( "default" => "#f27649", "hover" => "#ef825a" ),
      array( "default" => "#154659", "hover" => "#235b70" ),
      array( "default" => "#59535e", "hover" => "#66606b" ),
      array( "default" => "#a12858", "hover" => "#b03c6a" ),
      array( "default" => "#ffc000", "hover" => "#feca53" ),
      array( "default" => "#097609", "hover" => "#218f21" )
  );

  public function __construct() {
    $this->_setDependencies();
    $this->_includeModelsAndEntities();

    register_activation_hook(__FILE__, array($this, '_internalActivationHook'));
    add_action( "init", array($this, "_initHook"));
    add_action( 'admin_menu', array( $this, '_menuHook' ) );
    add_action( 'admin_enqueue_scripts', array($this, '_assetsHook') );
    add_action( 'wp_ajax_' . $this->pluginActionPrefix . 'GetTitleList',
        array($this, '_ajaxGetPostTitleList') );
    add_action( 'wp_footer', array( $this, 'footerScriptTracking'), 100 );
  }

  private function _setDependencies() {
    $this->pluginFilePath = dirname(__FILE__) . DIRECTORY_SEPARATOR;
    $this->pluginURLPath  = plugin_dir_url(__FILE__);
  }

  private function _includeModelsAndEntities() {
    require_once($this->pluginFilePath . 'model/versionControl.php');
    require_once($this->pluginFilePath . 'model/backendRequest.php');
    require_once($this->pluginFilePath . 'model/settings.php');
    require_once($this->pluginFilePath . 'model/titleHandler.php');
    require_once($this->pluginFilePath . 'model/imageHandler.php');
    require_once($this->pluginFilePath . 'model/excerptHandler.php');
    require_once($this->pluginFilePath . 'model/postWidget.php');

    require_once($this->pluginFilePath . 'entity/_abstractDatabase.php');
    require_once($this->pluginFilePath . 'entity/postAssign.php');
    require_once($this->pluginFilePath . 'entity/postDisplay.php');
    require_once($this->pluginFilePath . 'entity/postTitle.php');
    require_once($this->pluginFilePath . 'entity/postImageAssign.php');
    require_once($this->pluginFilePath . 'entity/postImageDisplay.php');
    require_once($this->pluginFilePath . 'entity/postImage.php');
    require_once($this->pluginFilePath . 'entity/postExcerptAssign.php');
    require_once($this->pluginFilePath . 'entity/postExcerptDisplay.php');
    require_once($this->pluginFilePath . 'entity/postExcerpt.php');
  }

  public function _menuHook() {
    add_menu_page(
        $this->pluginName,
        $this->pluginName,
        'manage_options',
        $this->pluginSlug,
        array(
            $this, 'displayAdministration'
        ),
        $this->pluginURLPath . 'assets/icon.png'
    );
  }

  public function _initHook() {
    $this->_initModelsAndEntities();
  }

  private function _initModelsAndEntities() {
    $this->entityPostAssign      = new ContentEngagePostAssign();
    $this->entityPostDisplay     = new ContentEngagePostDisplay();
    $this->entityPostTitle       = new ContentEngagePostTitle();

    $this->entityPostImageAssign   = new ContentEngagePostImageAssign();
    $this->entityPostImageDisplay  = new ContentEngagePostImageDisplay();
    $this->entityPostImage         = new ContentEngagePostImage();

    $this->entityPostExcerptAssign  = new ContentEngagePostExcerptAssign();
    $this->entityPostExcerptDisplay = new ContentEngagePostExcerptDisplay();
    $this->entityPostExcerpt        = new ContentEngagePostExcerpt();

    $this->modelVersionControl   = new ContentEngageVersionControl();
    $this->modelBackendRequest   = new ContentEngageBackendRequest();
    $this->modelSettings         = new ContentEngageSettings();
    $this->modelTitleHandler     = new ContentEngageTitleHandler();
    $this->modelImageHandler     = new ContentEngageImageHandler();
    $this->modelExcerptHandler   = new ContentEngageExcerptHandler();
    $this->modelPostWidget       = new ContentEngagePostWidget();
  }

  public function _ajaxGetPostTitleList() {

  }

  public function displayAdministration() {
    $this->modelBackendRequest->process();

    echo '<div class="bootstrap_environment">';
    require_once('views/_header.php');

    if(isset($_GET['sub-page']) && $_GET['sub-page'] == 'settings')
      require('views/settings.php');
    else if(isset($_GET['sub-page']) && $_GET['sub-page'] == 'support')
      require('views/support.php');
    else if(isset($_GET['sub-page']) && $_GET['sub-page'] == 'view')
      require('views/view.php');
    else if(isset($_GET['sub-page']) && $_GET['sub-page'] == 'view-chart')
      require('views/view-chart.php');
    else if(isset($_GET['sub-page']) && $_GET['sub-page'] == 'view-image')
      require('views/view-image.php');
    else if(isset($_GET['sub-page']) && $_GET['sub-page'] == 'view-excerpt')
      require('views/view-excerpt.php');
    else if(isset($_GET['sub-page']) && $_GET['sub-page'] == 'license')
      require('views/license.php');
    else if(isset($_GET['sub-page']) && $_GET['sub-page'] == 'documentation')
      require('views/documentation.php');
    else
      require('views/index.php');
    echo '</div>';
  }

  public function _assetsHook($hook) {
    $this->_postPageScripts($hook);

    if(!(isset($_GET['page']) && $_GET['page'] == $this->pluginSlug))
      return;

    wp_enqueue_style( 'hint.css', plugins_url('assets/hint.css', __FILE__ ));

    wp_enqueue_script(
        'bootstrap',
        plugins_url('assets/bootstrap.min.js', __FILE__ ),
        array( 'jquery'), false, true
    );
    wp_enqueue_script(
        'chart.js',
        plugins_url('assets/Chart.min.js', __FILE__ ),
        array( 'jquery'), false, true
    );

    wp_enqueue_style( 'bootstrap', plugins_url('assets/admin-style-bootstrap.css', __FILE__) );
    wp_enqueue_style( $this->pluginSlug . '-admin', plugins_url('assets/admin-style.css', __FILE__) );
  }

  private function _postPageScripts($hook) {
    if ( $hook == 'post-new.php' || $hook == 'post.php' ) {
      global $post;

      wp_enqueue_script('jquery');
      wp_enqueue_script(
          $this->pluginSlug . '-post-backend',
          plugins_url('assets/post-backend.js', __FILE__ ),
          array( 'jquery'), false, true
      );
      wp_localize_script(
          $this->pluginSlug . '-post-backend',
          'titleList',
          $this->entityPostTitle->getAllByPostID($post->ID)
      );
      wp_enqueue_style(
          $this->pluginSlug . '-post-image',
          plugins_url('assets/post-backend.css', __FILE__ )
      );

      wp_enqueue_script(
          $this->pluginSlug . '-post-backend-image',
          plugins_url('assets/post-image-backend.js', __FILE__ ),
          array( 'jquery'), false, true
      );
      wp_localize_script(
          $this->pluginSlug . '-post-backend-image',
          'imageList',
          $this->entityPostImage->getAllByPostID($post->ID)
      );
      wp_enqueue_style(
          $this->pluginSlug . '-post-image-backend',
          plugins_url('assets/post-image-backend.css', __FILE__ )
      );

      wp_enqueue_script(
          $this->pluginSlug . '-post-backend-excerpt',
          plugins_url('assets/post-excerpt-backend.js', __FILE__ ),
          array( 'jquery'), false, true
      );
      wp_localize_script(
          $this->pluginSlug . '-post-backend-excerpt',
          'excerptList',
          $this->entityPostExcerpt->getAllByPostID($post->ID)
      );
      wp_enqueue_style(
          $this->pluginSlug . '-post-excerpt-backend',
          plugins_url('assets/post-excerpt-backend.css', __FILE__ )
      );
    }
  }

  public function _internalActivationHook() {
    global $wpdb;

    $query = file_get_contents(dirname(__FILE__) . '/install.sql');

    $query = str_replace('content_engage_' ,
        $wpdb->base_prefix . 'content_engage_',
        $query);

    $queries = explode(';', $query);


    foreach($queries as $query)
      if(strlen($query)> 20)
        $response = $wpdb->query($query);

    if(!class_exists("ContentEngageVersionControl"))
      require_once($this->pluginFilePath . 'model/versionControl.php');

    $inactiveVersionControl   = new ContentEngageVersionControl(false);

    $inactiveVersionControl->setCurrentDatabaseVersion(
        $inactiveVersionControl->currentDatabaseVersion
    );
  }

  public function footerScriptTracking() {
    if( $this->footerScript != '' )
      echo '<script type="text/javascript">' . $this->footerScript . '</script>';
  }

}

ContentEngageController::getInstance();