<?php

/**
 * Class ContentEngagePostExcerpt
 * @author Andrei-Robert Rusu
 */
class ContentEngagePostExcerpt extends ContentEngageAbstractDatabase {

  private $_table_name = 'content_engage_post_excerpt';

  public function init() {
    $this->_table_name  = $this->wp_db->base_prefix . $this->_table_name;
  }

  public function getTableName() {
    return $this->_table_name;
  }

  public function getAll() {
    $sql = 'SELECT display.*
                   FROM `' . $this->_table_name . '` display';

    $information = $this->wp_db->get_results($sql);

    return $information === null ? array() : $information;
  }

  public function getActiveDateList($postID) {
    $sql = 'SELECT DATE(creation_date) as date
              FROM ' . ContentEngageController::getInstance()->entityPostExcerptDisplay->getTableName() . '
              WHERE post_id = ' . $postID . '
              GROUP BY date DESC
           ';

    $information = $this->wp_db->get_results($sql);
    $ret         = array();

    if($information !== null)
      foreach($information as $i)
        $ret[] = $i->date;

    return $ret;
  }

  public function getPostExcerptListWithInformation($postID) {
    $entityPostExcerptDisplayTableName = ContentEngageController::getInstance()->entityPostExcerptDisplay->getTableName();
    $entityPostExcerptAssignTableName  = ContentEngageController::getInstance()->entityPostExcerptAssign->getTableName();

    $sql = 'SELECT id,
                   excerpt,
                   IFNULL((
                     SELECT COUNT(DISTINCT`' . $entityPostExcerptDisplayTableName . '`.`ip_address`)
                       FROM `' . $entityPostExcerptDisplayTableName . '`
                      WHERE `' . $this->getTableName() . '`.`id` = `' . $entityPostExcerptDisplayTableName . '`.`post_excerpt_id`
                   ), 0) as uniqueDisplays,
                   IFNULL((
                     SELECT COUNT(DISTINCT`' . $entityPostExcerptAssignTableName . '`.`ip_address`)
                       FROM `' . $entityPostExcerptAssignTableName . '`
                      WHERE `' . $this->getTableName() . '`.`id` = `' . $entityPostExcerptAssignTableName . '`.`post_excerpt_id`
                   ), 0) as uniqueAssigns,
                   ( SELECT COUNT(`' . $entityPostExcerptDisplayTableName . '`.`id`)
                       FROM `' . $entityPostExcerptDisplayTableName . '`
                      WHERE `' . $this->getTableName() . '`.`id` = `' . $entityPostExcerptDisplayTableName . '`.`post_excerpt_id`
                   ) as displays,
                   ( SELECT COUNT(`' . $entityPostExcerptAssignTableName . '`.`id`)
                       FROM `' . $entityPostExcerptAssignTableName . '`
                      WHERE `' . $this->getTableName() . '`.`id` = `' . $entityPostExcerptAssignTableName . '`.`post_excerpt_id`
                   ) as assigns
              FROM `' . $this->getTableName() . '`
             WHERE post_id = ' . $postID . '
           ';

    $information = $this->wp_db->get_results($sql);

    return $information === null ? array() : $information;
  }

  public function getPostExcerptListWithInformationByDate($postID, $date) {
    $entityPostExcerptDisplayTableName = ContentEngageController::getInstance()->entityPostExcerptDisplay->getTableName();
    $entityPostExcerptAssignTableName  = ContentEngageController::getInstance()->entityPostExcerptAssign->getTableName();

    $sql = 'SELECT id,
                   excerpt,
                   IFNULL((
                     SELECT COUNT(DISTINCT`' . $entityPostExcerptDisplayTableName . '`.`ip_address`)
                       FROM `' . $entityPostExcerptDisplayTableName . '`
                      WHERE `' . $this->getTableName() . '`.`id` = `' . $entityPostExcerptDisplayTableName . '`.`post_excerpt_id`
                        AND DATE(`' . $entityPostExcerptDisplayTableName . '`.`creation_date`) = "' . $date . '"
                   ), 0) as uniqueDisplays,
                   ( SELECT COUNT(`' . $entityPostExcerptDisplayTableName . '`.`id`)
                       FROM `' . $entityPostExcerptDisplayTableName . '`
                      WHERE `' . $this->getTableName() . '`.`id` = `' . $entityPostExcerptDisplayTableName . '`.`post_excerpt_id`
                        AND DATE(`' . $entityPostExcerptDisplayTableName . '`.`creation_date`) = "' . $date . '"
                   ) as displays,
                   ( SELECT COUNT(`' . $entityPostExcerptAssignTableName . '`.`id`)
                       FROM `' . $entityPostExcerptAssignTableName . '`
                      WHERE `' . $this->getTableName() . '`.`id` = `' . $entityPostExcerptAssignTableName . '`.`post_excerpt_id`
                        AND DATE(`' . $entityPostExcerptAssignTableName . '`.`creation_date`) = "' . $date . '"
                   ) as assigns
              FROM `' . $this->getTableName() . '`
             WHERE post_id = ' . $postID . '
           ';

    $information = $this->wp_db->get_results($sql);

    return $information === null ? array() : $information;
  }

  public function getExcerptList() {
    $sql = 'SELECT post_id as post_id,
                   excerpt as excerpt,
                   SUM(displays) as displays,
                   SUM(assigns) as assigns
              FROM wp_content_engage_post_excerpt
              GROUP BY post_id';

    $information = $this->wp_db->get_results($sql);

    return $information === null ? array() : $information;
  }

  public function getAllByPostID($postID) {
    $sql = 'SELECT *
                   FROM `' . $this->_table_name . '`
                   WHERE post_id = "' . intval($postID) . '"';

    $information = $this->wp_db->get_results($sql);

    return $information === null ? array() : $information;
  }

  public function getNaturalDeliveryExcerptBySuccess($postID) {
    $entityPostExcerptDisplayTableName = ContentEngageController::getInstance()->entityPostExcerptDisplay->getTableName();
    $entityPostExcerptAssignTableName = ContentEngageController::getInstance()->entityPostExcerptAssign->getTableName();

    $sql = 'SELECT *,
                   IFNULL((
                     SELECT COUNT(DISTINCT`' . $entityPostExcerptDisplayTableName . '`.`ip_address`)
                       FROM `' . $entityPostExcerptDisplayTableName . '`
                      WHERE `' . $this->getTableName() . '`.`id` = `' . $entityPostExcerptDisplayTableName . '`.`post_excerpt_id`
                   ), 1) as uniqueDisplays,
                   IFNULL((
                     SELECT COUNT(DISTINCT`' . $entityPostExcerptAssignTableName . '`.`ip_address`)
                       FROM `' . $entityPostExcerptAssignTableName . '`
                      WHERE `' . $this->getTableName() . '`.`id` = `' . $entityPostExcerptAssignTableName . '`.`post_excerpt_id`
                   ), 1) as uniqueAssigns
                   FROM `' . $this->_table_name . '`
                   WHERE post_id = "' . intval($postID) . '"
                   ORDER BY uniqueAssigns / uniqueDisplays ASC';

    $information = $this->wp_db->get_row($sql);

    return $information === null ? array() : $information;
  }

  public function getNaturalDeliveryExcerptByAssignCount($postID) {
    $sql = 'SELECT *
                   FROM `' . $this->_table_name . '`
                   WHERE post_id = "' . intval($postID) . '"
                   ORDER BY assigns ASC';

    $information = $this->wp_db->get_row($sql);

    return $information === null ? array() : $information;
  }

  public function getById($id) {
    $sql = 'SELECT *
                   FROM `' . $this->_table_name . '`
                   WHERE id = "' . intval($id) . '"';

    $information = $this->wp_db->get_row($sql);

    return $information === null ? false : $information;
  }

  public function increaseAssignCountByID($id) {
    $this->increaseFieldValue('assigns', 1, array('id' => $id));
  }

  public function increaseDisplayCountByID($id) {
    $this->increaseFieldValue('displays', 1, array('id' => $id));
  }

  public function cleanDeleteByID($excerptID) {
    $this->delete(array("id" => $excerptID));
    ContentEngageController::getInstance()->entityPostExcerptAssign->delete(
        array("post_excerpt_id" => $excerptID)
    );
    ContentEngageController::getInstance()->entityPostExcerptDisplay->delete(
        array("post_excerpt_id" => $excerptID)
    );
  }

  public function deleteAllByPostID($campaignID) {
    $this->delete(array('post_id' => $campaignID));
  }

}