<?php

/**
 * Class ContentEngagePostImage
 * @author Andrei-Robert Rusu
 */
class ContentEngagePostImage extends ContentEngageAbstractDatabase {

  private $_table_name = 'content_engage_post_image';

  public function init() {
    $this->_table_name  = $this->wp_db->base_prefix . $this->_table_name;
  }

  public function getTableName() {
    return $this->_table_name;
  }

  public function getAll() {
    $sql = 'SELECT display.*
                   FROM `' . $this->_table_name . '` display';

    $information = $this->wp_db->get_results($sql);

    return $information === null ? array() : $information;
  }

  public function getFirstByPostID($postID) {
    $sql = 'SELECT *
                   FROM `' . $this->_table_name . '`
                   WHERE post_id = ' . intval($postID) . '
                   ORDER BY id ASC
           ';

    $information = $this->wp_db->get_row($sql);

    return $information === null ? false : $information;
  }

  public function getActiveDateList($postID) {
    $sql = 'SELECT DATE(creation_date) as date
              FROM ' . ContentEngageController::getInstance()->entityPostImageDisplay->getTableName() . '
              WHERE post_id = ' . $postID . '
              GROUP BY date DESC
           ';

    $information = $this->wp_db->get_results($sql);
    $ret         = array();

    if($information !== null)
      foreach($information as $i)
        $ret[] = $i->date;

    return $ret;
  }

  public function getPostImageListWithInformation($postID) {
    $entityPostDisplayTableName = ContentEngageController::getInstance()->entityPostImageDisplay->getTableName();
    $entityPostAssignTableName  = ContentEngageController::getInstance()->entityPostImageAssign->getTableName();

    $sql = 'SELECT id,
                   image,
                   IFNULL((
                     SELECT COUNT(DISTINCT`' . $entityPostDisplayTableName . '`.`ip_address`)
                       FROM `' . $entityPostDisplayTableName . '`
                      WHERE `' . $this->getTableName() . '`.`id` = `' . $entityPostDisplayTableName . '`.`post_image_id`
                   ), 0) as uniqueDisplays,
                   IFNULL((
                     SELECT COUNT(DISTINCT`' . $entityPostAssignTableName . '`.`ip_address`)
                       FROM `' . $entityPostAssignTableName . '`
                      WHERE `' . $this->getTableName() . '`.`id` = `' . $entityPostAssignTableName . '`.`post_image_id`
                   ), 0) as uniqueAssigns,
                   ( SELECT COUNT(`' . $entityPostDisplayTableName . '`.`id`)
                       FROM `' . $entityPostDisplayTableName . '`
                      WHERE `' . $this->getTableName() . '`.`id` = `' . $entityPostDisplayTableName . '`.`post_image_id`
                   ) as displays,
                   ( SELECT COUNT(`' . $entityPostAssignTableName . '`.`id`)
                       FROM `' . $entityPostAssignTableName . '`
                      WHERE `' . $this->getTableName() . '`.`id` = `' . $entityPostAssignTableName . '`.`post_image_id`
                   ) as assigns
              FROM `' . $this->getTableName() . '`
             WHERE post_id = ' . $postID . '
           ';

    $information = $this->wp_db->get_results($sql);

    return $information === null ? array() : $information;
  }

  public function getPostImageListWithInformationByDate($postID, $date) {
    $entityPostDisplayTableName = ContentEngageController::getInstance()->entityPostImageDisplay->getTableName();
    $entityPostAssignTableName  = ContentEngageController::getInstance()->entityPostImageAssign->getTableName();

    $sql = 'SELECT id,
                   image,
                   IFNULL((
                     SELECT COUNT(DISTINCT`' . $entityPostDisplayTableName . '`.`ip_address`)
                       FROM `' . $entityPostDisplayTableName . '`
                      WHERE `' . $this->getTableName() . '`.`id` = `' . $entityPostDisplayTableName . '`.`post_image_id`
                        AND DATE(`' . $entityPostDisplayTableName . '`.`creation_date`) = "' . $date . '"
                   ), 0) as uniqueDisplays,
                   ( SELECT COUNT(`' . $entityPostDisplayTableName . '`.`id`)
                       FROM `' . $entityPostDisplayTableName . '`
                      WHERE `' . $this->getTableName() . '`.`id` = `' . $entityPostDisplayTableName . '`.`post_image_id`
                        AND DATE(`' . $entityPostDisplayTableName . '`.`creation_date`) = "' . $date . '"
                   ) as displays,
                   ( SELECT COUNT(`' . $entityPostAssignTableName . '`.`id`)
                       FROM `' . $entityPostAssignTableName . '`
                      WHERE `' . $this->getTableName() . '`.`id` = `' . $entityPostAssignTableName . '`.`post_image_id`
                        AND DATE(`' . $entityPostAssignTableName . '`.`creation_date`) = "' . $date . '"
                   ) as assigns
              FROM `' . $this->getTableName() . '`
             WHERE post_id = ' . $postID . '
           ';

    $information = $this->wp_db->get_results($sql);

    return $information === null ? array() : $information;
  }

  public function getImageList() {
    $sql = 'SELECT post_id as post_id,
                   image as image,
                   SUM(displays) as displays,
                   SUM(assigns) as assigns
              FROM wp_content_engage_post_image
              GROUP BY post_id';

    $information = $this->wp_db->get_results($sql);

    return $information === null ? array() : $information;
  }

  public function getAllByPostID($postID) {
    $sql = 'SELECT *
                   FROM `' . $this->_table_name . '`
                   WHERE post_id = "' . intval($postID) . '"';

    $information = $this->wp_db->get_results($sql);

    return $information === null ? array() : $information;
  }

  public function getNaturalDeliveryImageBySuccess($postID) {
    $entityPostDisplayTableName = ContentEngageController::getInstance()->entityPostImageDisplay->getTableName();
    $entityPostAssignTableName = ContentEngageController::getInstance()->entityPostImageAssign->getTableName();

    $sql = 'SELECT *,
                   IFNULL((
                     SELECT COUNT(DISTINCT`' . $entityPostDisplayTableName . '`.`ip_address`)
                       FROM `' . $entityPostDisplayTableName . '`
                      WHERE `' . $this->getTableName() . '`.`id` = `' . $entityPostDisplayTableName . '`.`post_image_id`
                   ), 1) as uniqueDisplays,
                   IFNULL((
                     SELECT COUNT(DISTINCT`' . $entityPostAssignTableName . '`.`ip_address`)
                       FROM `' . $entityPostAssignTableName . '`
                      WHERE `' . $this->getTableName() . '`.`id` = `' . $entityPostAssignTableName . '`.`post_image_id`
                   ), 1) as uniqueAssigns
                   FROM `' . $this->_table_name . '`
                   WHERE post_id = "' . intval($postID) . '"
                   ORDER BY uniqueAssigns / uniqueDisplays ASC';

    $information = $this->wp_db->get_row($sql);

    return $information === null ? array() : $information;
  }

  public function getNaturalDeliveryImageByAssignCount($postID) {
    $sql = 'SELECT *
                   FROM `' . $this->_table_name . '`
                   WHERE post_id = "' . intval($postID) . '"
                   ORDER BY assigns ASC';

    $information = $this->wp_db->get_row($sql);

    return $information === null ? array() : $information;
  }

  public function getById($id) {
    $sql = 'SELECT *
                   FROM `' . $this->_table_name . '`
                   WHERE id = "' . intval($id) . '"';

    $information = $this->wp_db->get_row($sql);

    return $information === null ? false : $information;
  }

  public function increaseAssignCountByID($id) {
    $this->increaseFieldValue('assigns', 1, array('id' => $id));
  }

  public function increaseDisplayCountByID($id) {
    $this->increaseFieldValue('displays', 1, array('id' => $id));
  }

  public function cleanDeleteByID($imageID) {
    $this->delete(array("id" => $imageID));
    ContentEngageController::getInstance()->entityPostImageAssign->delete(
        array("post_image_id" => $imageID)
    );
    ContentEngageController::getInstance()->entityPostImageDisplay->delete(
        array("post_image_id" => $imageID)
    );
  }

  public function deleteAllByPostID($campaignID) {
    $this->delete(array('post_id' => $campaignID));
  }

}