<?php

/**
 * Class ContentEngageBackendRequest
 * @author Andrei-Robert Rusu
 */
class ContentEngageBackendRequest {

  public $notifications              = array(
    'errors'        => array(),
    'notifications' => array(),
    'success'       => array(),
    'warnings'      => array()
  );
  public $actions                    = array();
  public $_processRequestComplete    = 0;

  public function __construct() {
    add_action( 'save_post', array($this, '_handlePostSave') );
  }

  public function process() {
    if(!is_admin())
      return;

    $this->_init();
  }

  public function _init() {
    if(isset($_POST[ContentEngageController::getInstance()->pluginPrefix . '-save-settings'])) {
      ContentEngageController::getInstance()->modelSettings->setHomepageTestMode(intval($_POST['homepage_test_mode']));
      ContentEngageController::getInstance()->modelSettings->setAutomaticTitleAssociation(intval($_POST['automatic_title_association']));
      ContentEngageController::getInstance()->modelSettings->setAutomaticTitleAssociationMode($_POST['automatic_title_association_mode']);
      ContentEngageController::getInstance()->modelSettings->setAutomaticExcerptAssociation(intval($_POST['automatic_excerpt_association']));
      ContentEngageController::getInstance()->modelSettings->setDefaultSplitTestingMargin(intval($_POST['default_split_testing_margin']));

      $this->notifications['success'][] = __("Your settings have been updated");
    }
  }

  public function _handlePostSave($post_id) {
    if(isset($_POST['content_engage_field_new']))
      foreach($_POST['content_engage_field_new'] as $title)
        if(trim($title) != '')
          ContentEngageController::getInstance()->entityPostTitle->insert(array(
            'title'   => $title,
            'post_id' => $post_id
          ));

    if(isset($_POST['content_engage_field_existent'])) {
      foreach($_POST['content_engage_field_existent'] as $titleID => $title) {
        if(trim($title) != '') {
          ContentEngageController::getInstance()->entityPostTitle->update(array(
              'title'   => $title
          ), array(
              'id'      => $titleID
          ));
        } else {
          ContentEngageController::getInstance()->entityPostTitle->cleanDeleteByID($titleID);
        }
      }
    }

    if(isset($_POST['content_engage_image_field_new']))
      foreach($_POST['content_engage_image_field_new'] as $image)
        if(trim($image) != '')
          ContentEngageController::getInstance()->entityPostImage->insert(array(
              'image'   => $image,
              'post_id' => $post_id
          ));

    if(isset($_POST['content_engage_image_field_existent'])) {
      foreach($_POST['content_engage_image_field_existent'] as $titleID => $image) {
        if(trim($image) != '') {
          ContentEngageController::getInstance()->entityPostImage->update(array(
              'image'   => $image
          ), array(
              'id'      => $titleID
          ));
        } else {
          ContentEngageController::getInstance()->entityPostImage->cleanDeleteByID($titleID);
        }
      }
    }

    if(isset($_POST['content_engage_excerpt_field_new']))
      foreach($_POST['content_engage_excerpt_field_new'] as $image)
        if(trim($image) != '')
          ContentEngageController::getInstance()->entityPostExcerpt->insert(array(
              'content' => $image,
              'post_id' => $post_id
          ));

    if(isset($_POST['content_engage_excerpt_field_existent'])) {
      foreach($_POST['content_engage_excerpt_field_existent'] as $titleID => $image) {
        if(trim($image) != '') {
          ContentEngageController::getInstance()->entityPostExcerpt->update(array(
              'content' => $image
          ), array(
              'id'      => $titleID
          ));
        } else {
          ContentEngageController::getInstance()->entityPostExcerpt->cleanDeleteByID($titleID);
        }
      }
    }
  }

  public function displayRequestNotifications() {
    if(isset($this->notifications['errors']))
      foreach($this->notifications['errors'] as $error)
        echo '<div class="alert alert-danger">' . $error . '</div>';

    if(isset($this->notifications['notifications']))
      foreach($this->notifications['notifications'] as $notification)
        echo '<div class="alert alert-info">' . $notification . '</div>';

    if(isset($this->notifications['success']))
      foreach($this->notifications['success'] as $success)
        echo '<div class="alert alert-success">' . $success . '</div>';

    if(isset($this->notifications['warnings']))
      foreach($this->notifications['warnings'] as $warning)
        echo '<div class="alert alert-warning">' . $warning . '</div>';
  }

}