<?php

/**
 * Class ContentEngageExcerptHandler
 * @author Andrei-Robert Rusu
 */
class ContentEngageExcerptHandler {

  public $excerptVariationIdentifier      = 'cee';
  public $postID;

  public $assignedExcerpt = '';
  public $displayExcerptSave = array();

  public $deliveredVerdict = false;

  public function __construct() {
    if(is_admin())
      return;

    $this->_setPostID();

    add_filter( 'get_the_excerpt', array($this, '_handlePostExcerpt'), 1, 2 );

    if($this->postID == 0)
      return;

    if(!(ContentEngageController::getInstance()->modelTitleHandler->verdictExists($this->postID)
          && ContentEngageController::getInstance()->modelImageHandler->verdictExists($this->postID)))
      return;

    if($this->_deliverCampaignVerdictIfExists())
      return;

    if(isset($_GET[$this->excerptVariationIdentifier])) {
      $this->_deliverVariationByInformationObject(
          $this->_simpleHexDecodingWithStrRot13($_GET[$this->excerptVariationIdentifier])
      );

      return;
    }

    if($this->_deliverPreviouslyDelivered())
      return;

    if($this->_deliverPreviouslyAssigned())
      return;

    if($this->_assignAndRedirectToHeadline())
      return;
  }

  private function _deliverVariationByInformationObject($excerptVariationID, $registerDisplay = true) {
    $excerptVariationInformation = ContentEngageController::getInstance()->entityPostExcerpt->getById($excerptVariationID);

    $this->assignedExcerpt = $excerptVariationInformation->content;

    if($registerDisplay)
      $this->_registerDisplay($excerptVariationInformation->id);
  }

  private function _deliverCampaignVerdictIfExists() {
    if($this->verdictExists()) {
      $nextPostExcerptObject = ContentEngageController::getInstance()->entityPostExcerpt->getNaturalDeliveryExcerptBySuccess(
          $this->postID
      );

      if(empty($nextPostExcerptObject))
        return false;

      $this->_deliverVariationByInformationObject($nextPostExcerptObject->id, false);

      $this->deliveredVerdict = true;
      return true;
    }

    return false;
  }

  private function _deliverPreviouslyDelivered() {
    $latestDelivered = ContentEngageController::getInstance()->entityPostExcerptDisplay->getNewestByPostIdAndIPAddress(
        $this->postID, $_SERVER['REMOTE_ADDR']
    );

    if($latestDelivered !== false) {
      $currentPage     = $this->_getCurrentPage();
      $redirectionPage = $currentPage . (strpos($currentPage, "?") === false ? '?' : '&') .
          $this->excerptVariationIdentifier . '=' .
          $this->_simpleHexEncodingWithStrRot13($latestDelivered->post_excerpt_id);

      wp_redirect($redirectionPage);
      exit;
    }

    return false;
  }

  private function _deliverPreviouslyAssigned() {
    $latestAssigned = ContentEngageController::getInstance()->entityPostExcerptAssign->getNewestByPostIdAndIPAddress(
        $this->postID, $_SERVER['REMOTE_ADDR']
    );

    if($latestAssigned !== false) {
      $currentPage     = $this->_getCurrentPage();
      $redirectionPage = $currentPage . (strpos($currentPage, "?") === false ? '?' : '&') .
          $this->excerptVariationIdentifier . '=' .
          $this->_simpleHexEncodingWithStrRot13($latestAssigned->post_excerpt_id);

      wp_redirect($redirectionPage);
      exit;
    }

    return false;
  }

  private function _assignAndRedirectToHeadline() {
    $nextPostExcerptObject = ContentEngageController::getInstance()->entityPostExcerpt->getNaturalDeliveryExcerptByAssignCount(
        $this->postID
    );

    if(empty($nextPostExcerptObject))
      return false;


    $this->_registerAssignment($nextPostExcerptObject->id);

    $currentPage     = $this->_getCurrentPage();
    $redirectionPage = $currentPage . (strpos($currentPage, "?") === false ? '?' : '&') .
        $this->excerptVariationIdentifier . '=' .
        $this->_simpleHexEncodingWithStrRot13($nextPostExcerptObject->id);

    wp_redirect($redirectionPage);
    exit;
  }

  public function _handlePostExcerpt($excerpt, $id = null) {
    if(isset($this->displayExcerptSave[$id]))
      return $this->displayExcerptSave[$id];

    global $post;

    if(!isset($post->ID))
      return $excerpt;

    $id = $post->ID;

    if($id != $this->postID) {
      if($this->verdictExists($id)) {
        $nextPostExcerptObject = ContentEngageController::getInstance()->entityPostExcerpt->getNaturalDeliveryExcerptBySuccess(
            $id
        );

        if(!empty($nextPostExcerptObject)) {
          $this->displayExcerptSave[$id] = $nextPostExcerptObject->content;
          return $nextPostExcerptObject->content;
        }
      }


      $latestDelivered = ContentEngageController::getInstance()->entityPostExcerptDisplay->getNewestByPostIdAndIPAddress(
          $id, $_SERVER['REMOTE_ADDR']
      );

      if($latestDelivered !== false) {
        $nextPostExcerptObject = ContentEngageController::getInstance()->entityPostExcerpt->getById($latestDelivered->post_excerpt_id);

        if(!empty($nextPostExcerptObject)) {
          $this->displayExcerptSave[$id] = $nextPostExcerptObject->content;
          return $nextPostExcerptObject->content;
        }
      }

      $latestAssigned = ContentEngageController::getInstance()->entityPostExcerptAssign->getNewestByPostIdAndIPAddress(
          $id, $_SERVER['REMOTE_ADDR']
      );

      if($latestAssigned !== false) {
        $nextPostExcerptObject = ContentEngageController::getInstance()->entityPostExcerpt->getById($latestAssigned->post_excerpt_id);

        if(!empty($nextPostExcerptObject)) {
          $this->displayExcerptSave[$id] = $nextPostExcerptObject->content;
          return $nextPostExcerptObject->content;
        }
      }

      if(ContentEngageController::getInstance()->modelSettings->automaticExcerptAssociation == 1) {
        $nextPostExcerptObject = ContentEngageController::getInstance()->entityPostExcerpt->getNaturalDeliveryExcerptByAssignCount(
            $id
        );

        if(!empty($nextPostExcerptObject)) {
          $this->_registerAssignment($nextPostExcerptObject->id, $id);
          $this->displayExcerptSave[$id] = $nextPostExcerptObject->content;
          return $nextPostExcerptObject->content;
        }
      }

      return $excerpt;
    }

    return ($this->assignedExcerpt == '' ? $excerpt : $this->assignedExcerpt);
  }

  private function _registerAssignment($postExcerptID, $postID = false) {
    ContentEngageController::getInstance()->entityPostExcerpt->increaseAssignCountByID($postExcerptID);
    ContentEngageController::getInstance()->entityPostExcerptAssign->insert(array(
        'post_excerpt_id' => $postExcerptID,
        'post_id'       => ($postID == false ? $this->postID : $postID),
        'ip_address'    => $_SERVER['REMOTE_ADDR']
    ));
  }

  private function _registerDisplay($postExcerptID) {
    ContentEngageController::getInstance()->entityPostExcerpt->increaseDisplayCountByID($postExcerptID);
    ContentEngageController::getInstance()->entityPostExcerptDisplay->insert(array(
        'post_excerpt_id' => $postExcerptID,
        'post_id'       => $this->postID,
        'ip_address'    => $_SERVER['REMOTE_ADDR']
    ));
  }

  private function _setPostID() {
    $this->postID = url_to_postid( $this->_getCurrentPage() );
  }

  private function _getCurrentPage() {
    return "http://".$_SERVER['SERVER_NAME'].$_SERVER['REQUEST_URI'];
  }

  /**
   * @param $token
   * @return string
   */
  private function _simpleHexEncodingWithStrRot13($token){
    return @str_rot13(bin2hex($token));
  }

  /**
   * @param $token
   * @return string
   */
  private function _simpleHexDecodingWithStrRot13($token){
    return @pack("H*",str_rot13($token));
  }

  public function verdictExists($postID = false) {
    $postID = $postID == false ? $this->postID : $postID;

    $uniqueDeliveries = ContentEngageController::getInstance()->entityPostExcerptAssign->getAllUniqueByPostID($postID);

    $splitTestMargin = get_post_meta($postID, ContentEngageController::getInstance()->postOptionSplitTestingMargin, true);
    $splitTestMargin = $splitTestMargin == '' ? ContentEngageController::getInstance()->modelSettings->defaultSplitTestingMargin : $splitTestMargin;

    return (count($uniqueDeliveries) > $splitTestMargin);
  }
}