<?php

/**
 * Class ContentEngageImageHandler
 * @author Andrei-Robert Rusu
 */
class ContentEngageImageHandler {

  public $imageVariationIdentifier      = 'cei';
  public $postID;

  public $assignedImageID = '';
  public $displayImageSave = array();

  public function __construct() {
    $this->_setPostID();

    add_filter( 'get_post_metadata', array($this, '_handlePostImage'), 10, 4 );

    if($this->postID == 0)
      return;

    if($this->_deliverCampaignVerdictIfExists())
      return;

    if(ContentEngageController::getInstance()->modelTitleHandler->deliveredVerdict == false)
      return;

    if(isset($_GET[$this->imageVariationIdentifier])) {
      $this->_deliverVariationByInformationObject(
          $this->_simpleHexDecodingWithStrRot13($_GET[$this->imageVariationIdentifier])
      );
      return;
    }

    if($this->_deliverPreviouslyDelivered())
      return;

    if($this->_assignAndRedirectToHeadline())
      return;
  }

  private function _deliverVariationByInformationObject($imageVariationID, $registerDisplay = true) {
    $imageVariationInformation = ContentEngageController::getInstance()->entityPostImage->getById($imageVariationID);

    $this->assignedImageID = $this->_getAttachmentID($imageVariationInformation->image);

    add_filter( 'get_post_metadata', array($this, '_handlePostImage'), 10, 4 );

    if($registerDisplay)
      $this->_registerDisplay($imageVariationInformation->id);
  }

  private function _deliverCampaignVerdictIfExists() {
    if($this->verdictExists()) {
      $nextPostImageObject = ContentEngageController::getInstance()->entityPostImage->getNaturalDeliveryImageBySuccess(
          $this->postID
      );

      if(empty($nextPostImageObject))
        return false;

      $this->_deliverVariationByInformationObject($nextPostImageObject->id, false);

      return true;
    }

    return false;
  }

  private function _deliverPreviouslyDelivered() {
    $latestDelivered = ContentEngageController::getInstance()->entityPostImageDisplay->getNewestByPostIdAndIPAddress(
        $this->postID, $_SERVER['REMOTE_ADDR']
    );

    if($latestDelivered !== false) {
      $currentPage     = $this->_getCurrentPage();
      $redirectionPage = $currentPage . (strpos($currentPage, "?") === false ? '?' : '&') .
          $this->imageVariationIdentifier . '=' .
          $this->_simpleHexEncodingWithStrRot13($latestDelivered->post_image_id);

      wp_redirect($redirectionPage);
      exit;
    }

    return false;
  }

  private function _assignAndRedirectToHeadline() {
    $nextPostImageObject = ContentEngageController::getInstance()->entityPostImage->getNaturalDeliveryImageByAssignCount(
        $this->postID
    );

    if(empty($nextPostImageObject))
      return false;


    $this->_registerAssignment($nextPostImageObject->id);

    $currentPage     = $this->_getCurrentPage();
    $redirectionPage = $currentPage . (strpos($currentPage, "?") === false ? '?' : '&') .
        $this->imageVariationIdentifier . '=' .
        $this->_simpleHexEncodingWithStrRot13($nextPostImageObject->id);

    wp_redirect($redirectionPage);
    exit;
  }

  public function _handlePostImage($metadata, $id, $meta_key, $single) {
    if($meta_key != '_thumbnail_id')
      return $metadata;

    if(!$single)
      return $metadata;

    if(isset($this->displayImageSave[$id]))
      return $this->displayImageSave[$id];

    if($id != $this->postID) {
      if($this->verdictExists($id)) {
        $nextPostImageObject = ContentEngageController::getInstance()->entityPostImage->getById($id);

        $postAttachmentID = $this->_getAttachmentID($nextPostImageObject->image);

        if(!empty($nextPostImageObject)) {
          $this->displayImageSave[$id] = $postAttachmentID;
          return $postAttachmentID;
        }
      }


      $latestDelivered = ContentEngageController::getInstance()->entityPostImageDisplay->getNewestByPostIdAndIPAddress(
          $id, $_SERVER['REMOTE_ADDR']
      );

      if($latestDelivered !== false) {
        $nextPostImageObject = ContentEngageController::getInstance()->entityPostImage->getById($latestDelivered->post_image_id);

        $postAttachmentID = $this->_getAttachmentID($nextPostImageObject->image);

        if(!empty($nextPostImageObject)) {
          $this->displayImageSave[$id] = $postAttachmentID;
          return $postAttachmentID;
        }
      }

      if($metadata == null) {
        $firstAvailableImage = ContentEngageController::getInstance()->entityPostImage->getFirstByPostID($id);

        if($firstAvailableImage != false) {
          $postAttachmentID = $this->_getAttachmentID($firstAvailableImage->image);

          $this->displayImageSave[$id] = $postAttachmentID;
          return $postAttachmentID;
        }
      }

      return $metadata;
    }

    return $this->assignedImageID;
  }

  private function _registerAssignment($postImageID) {
    ContentEngageController::getInstance()->entityPostImage->increaseAssignCountByID($postImageID);
    ContentEngageController::getInstance()->entityPostImageAssign->insert(array(
        'post_image_id' => $postImageID,
        'post_id'       => $this->postID,
        'ip_address'    => $_SERVER['REMOTE_ADDR']
    ));
  }

  private function _registerDisplay($postImageID) {
    ContentEngageController::getInstance()->entityPostImage->increaseDisplayCountByID($postImageID);
    ContentEngageController::getInstance()->entityPostImageDisplay->insert(array(
        'post_image_id' => $postImageID,
        'post_id'       => $this->postID,
        'ip_address'    => $_SERVER['REMOTE_ADDR']
    ));
  }

  private function _setPostID() {
    $this->postID = url_to_postid( $this->_getCurrentPage() );
  }

  private function _getCurrentPage() {
    return "http://".$_SERVER['SERVER_NAME'].$_SERVER['REQUEST_URI'];
  }

  /**
   * @param $token
   * @return string
   */
  private function _simpleHexEncodingWithStrRot13($token){
    return @str_rot13(bin2hex($token));
  }

  /**
   * @param $token
   * @return string
   */
  private function _simpleHexDecodingWithStrRot13($token){
    return @pack("H*",str_rot13($token));
  }

  private function _getAttachmentID($image_url) {
    global $wpdb;
    $attachment = $wpdb->get_col($wpdb->prepare("SELECT ID FROM $wpdb->posts WHERE guid='%s';", $image_url ));
    return $attachment[0];
  }

  public function verdictExists($postID = false) {
    $postID = $postID == false ? $this->postID : $postID;

    $uniqueDeliveries = ContentEngageController::getInstance()->entityPostImageAssign->getAllUniqueByPostID($postID);

    $splitTestMargin = get_post_meta($postID, ContentEngageController::getInstance()->postOptionSplitTestingMargin, true);
    $splitTestMargin = $splitTestMargin == '' ? ContentEngageController::getInstance()->modelSettings->defaultSplitTestingMargin : $splitTestMargin;

    return count($uniqueDeliveries) > $splitTestMargin;
  }
}