<?php

/**
 * Class ContentEngagePostWidget
 * @author Andrei-Robert Rusu
 */
class ContentEngagePostWidget {

  public $metaBoxId     = "";
  public $metaBoxName   = "ContentEngage Settings";
  public $postTypes     = array('post', 'page');

  public $fieldAliasPrefix            = '';
  public $fieldAliasNonce              = 'nonce';

  public function __construct() {
    add_action( 'init', array($this, 'init'));
    add_action( 'add_meta_boxes', array($this, 'registerMetaBoxes') );
    add_action( 'save_post', array($this, 'metaBoxSaveCallback') );
  }

  public function init() {
    $this->metaBoxId      = ContentEngageController::getInstance()->pluginSlug . '_post';

    $this->_fieldPrefixHandler();
  }

  private function _fieldPrefixHandler() {
    $this->fieldAliasPrefix  = str_replace("-", "_", ContentEngageController::getInstance()->pluginSlug);
    $this->fieldAliasNonce   = $this->fieldAliasPrefix . $this->fieldAliasNonce;
  }

  public function registerMetaBoxes() {
    foreach ( $this->postTypes as $postType )
      add_meta_box(
          $this->metaBoxId,
          __( $this->metaBoxName ),
          array($this, 'metaBoxDisplayCallback'),
          $postType,
          'side'
      );
  }

  public function metaBoxDisplayCallback($post) {
    wp_nonce_field( $this->fieldAliasPrefix, $this->fieldAliasNonce );

    $currentValue = get_post_meta( $post->ID, ContentEngageController::getInstance()->postOptionSplitTestingMargin, true );
    $testMode     = get_post_meta( $post->ID, ContentEngageController::getInstance()->postOptionTestMode, true );

    if($currentValue == "")
      $currentValue = ContentEngageController::getInstance()->modelSettings->defaultSplitTestingMargin;

    if($testMode == "")
      $testMode = 0;

    echo '<label for="' . ContentEngageController::getInstance()->postOptionSplitTestingMargin . '">';
    _e( 'Required number of Unique visitors to end split testing', 'myplugin_textdomain' );
    echo '</label> ';
    echo '<input type="text" class="widefat"
                             id="' . ContentEngageController::getInstance()->postOptionSplitTestingMargin .
                          '" name="' . ContentEngageController::getInstance()->postOptionSplitTestingMargin .
                          '" value="' . $currentValue . '"/>';
    echo '<p>After the goal is reached, the title with the best conversion rate will be automatically delivered</p>';

    echo '<label for="' . ContentEngageController::getInstance()->postOptionTestMode . '">';
    _e( 'Test Mode', 'myplugin_textdomain' );
    echo '</label> ';
    echo '<input type="checkbox" id="' . ContentEngageController::getInstance()->postOptionTestMode .
                '" name="' . ContentEngageController::getInstance()->postOptionTestMode . '"' .
                 ($testMode == 1 ? ' checked="checked" ' : '') .
                ' value="1"/>';
    echo '<p>This page will always display a random title while you view it.</p>';
  }

  public function metaBoxSaveCallback($post_id) {
    if ( ! isset( $_POST[$this->fieldAliasNonce] ) )
      return;
    if ( ! wp_verify_nonce( $_POST[$this->fieldAliasNonce], $this->fieldAliasPrefix ) )
      return;
    if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE )
      return;
    if ( isset( $_POST['post_type'] ) && 'page' == $_POST['post_type'] ) {
      if ( ! current_user_can( 'edit_page', $post_id ) )
        return;
    } else {
      if ( ! current_user_can( 'edit_post', $post_id ) )
        return;
    }

    if ( ! isset( $_POST[ContentEngageController::getInstance()->postOptionSplitTestingMargin] ) ) {
      return;
    }

    update_post_meta( $post_id, ContentEngageController::getInstance()->postOptionSplitTestingMargin, intval( $_POST[ContentEngageController::getInstance()->postOptionSplitTestingMargin] ) );
    update_post_meta( $post_id, ContentEngageController::getInstance()->postOptionTestMode, intval( isset($_POST[ContentEngageController::getInstance()->postOptionTestMode]) ? $_POST[ContentEngageController::getInstance()->postOptionTestMode] : 0 ) );
  }

}