<?php

/**
 * Class ContentEngageTitleHandler
 * @author Andrei-Robert Rusu
 */
class ContentEngageTitleHandler {

  public $trackingCookiePrefix          = 'cet_';
  public $titleVariationIdentifier      = 'cet';
  public $postID;

  public $assignedTitle = '';
  public $displayTitleSave = array();

  public $deliveredVerdict = false;

  public function __construct() {
    if(is_admin())
      return;

    $this->_setPostID();

    if($this->postID === 0 &&
        ContentEngageController::getInstance()->modelSettings->homepageTestMode) {
      add_filter( 'the_title', array($this, '_handlePostTitleTestMode'), 1, 2 );
      return;
    }

    add_filter( 'the_title', array($this, '_handlePostTitle'), 1, 2 );

    if($this->postID == 0)
      return;

    if($this->_handleTestMode())
      return;

    if($this->_deliverCampaignVerdictIfExists())
      return;

    if(isset($_GET[$this->titleVariationIdentifier])) {
      $this->_deliverVariationByInformationObject(
          $this->_simpleHexDecodingWithStrRot13($_GET[$this->titleVariationIdentifier])
      );

      return;
    }

    if($this->_deliverPreviouslyDelivered())
      return;

    if($this->_deliverPreviouslyAssigned())
      return;

    if($this->_deliverPreviouslyCookieAssigned())
      return;

    if($this->_assignAndRedirectToHeadline())
      return;
  }

  private function _deliverVariationByInformationObject($titleVariationID, $registerDisplay = true) {
    $titleVariationInformation = ContentEngageController::getInstance()->entityPostTitle->getById($titleVariationID);

    $this->assignedTitle = $titleVariationInformation->title;

    add_filter( 'wp_title',  array($this, '_handlePageTitle'), 1, 2 );

    if($registerDisplay)
      $this->_registerDisplay($titleVariationInformation->id);
  }

  private function _deliverCampaignVerdictIfExists() {
    if($this->verdictExists()) {
      $nextPostTitleObject = ContentEngageController::getInstance()->entityPostTitle->getNaturalDeliveryTitleBySuccess(
          $this->postID
      );

      if(empty($nextPostTitleObject))
        return false;

      $this->_deliverVariationByInformationObject($nextPostTitleObject->id, false);

      $this->deliveredVerdict = true;
      return true;
    }

    return false;
  }

  private function _deliverPreviouslyDelivered() {
    $latestDelivered = ContentEngageController::getInstance()->entityPostDisplay->getNewestByPostIdAndIPAddress(
        $this->postID, $_SERVER['REMOTE_ADDR']
    );

    if($latestDelivered !== false) {
      $currentPage     = $this->_getCurrentPage();
      $redirectionPage = $currentPage . (strpos($currentPage, "?") === false ? '?' : '&') .
          $this->titleVariationIdentifier . '=' .
          $this->_simpleHexEncodingWithStrRot13($latestDelivered->post_title_id);

      wp_redirect($redirectionPage);
      exit;
    }

    return false;
  }

  private function _deliverPreviouslyAssigned() {
    $latestAssigned = ContentEngageController::getInstance()->entityPostAssign->getNewestByPostIdAndIPAddress(
        $this->postID, $_SERVER['REMOTE_ADDR']
    );

    if($latestAssigned !== false) {
      $currentPage     = $this->_getCurrentPage();
      $redirectionPage = $currentPage . (strpos($currentPage, "?") === false ? '?' : '&') .
          $this->titleVariationIdentifier . '=' .
          $this->_simpleHexEncodingWithStrRot13($latestAssigned->post_title_id);

      wp_redirect($redirectionPage);
      exit;
    }

    return false;
  }

  private function _deliverPreviouslyCookieAssigned() {
    if(!isset($_COOKIE[$this->trackingCookiePrefix . $this->postID]))
      return false;

    $latestAssigned = ContentEngageController::getInstance()->entityPostTitle->getById(
        intval($_COOKIE[$this->trackingCookiePrefix . $this->postID])
    );

    if($latestAssigned !== false) {
      $this->_registerAssignment($latestAssigned->id);

      $currentPage     = $this->_getCurrentPage();
      $redirectionPage = $currentPage . (strpos($currentPage, "?") === false ? '?' : '&') .
          $this->titleVariationIdentifier . '=' .
          $this->_simpleHexEncodingWithStrRot13($latestAssigned->post_title_id);

      wp_redirect($redirectionPage);
      exit;
    }

    return false;
  }

  private function _assignAndRedirectToHeadline() {
    $nextPostTitleObject = ContentEngageController::getInstance()->entityPostTitle->getNaturalDeliveryTitleByAssignCount(
        $this->postID
    );

    if(empty($nextPostTitleObject))
      return false;


    $this->_registerAssignment($nextPostTitleObject->id);

    $currentPage     = $this->_getCurrentPage();
    $redirectionPage = $currentPage . (strpos($currentPage, "?") === false ? '?' : '&') .
        $this->titleVariationIdentifier . '=' .
        $this->_simpleHexEncodingWithStrRot13($nextPostTitleObject->id);

    wp_redirect($redirectionPage);
    exit;
  }

  public function _handlePostTitle($title, $id = null) {
    if(isset($this->displayTitleSave[$id]))
      return $this->displayTitleSave[$id];


    if($id != $this->postID) {
      if($this->verdictExists($id)) {
        $nextPostTitleObject = ContentEngageController::getInstance()->entityPostTitle->getNaturalDeliveryTitleBySuccess(
            $id
        );

        if(!empty($nextPostTitleObject)) {
          $this->displayTitleSave[$id] = $nextPostTitleObject->title;
          return $nextPostTitleObject->title;
        }
      }


      $latestDelivered = ContentEngageController::getInstance()->entityPostDisplay->getNewestByPostIdAndIPAddress(
          $id, $_SERVER['REMOTE_ADDR']
      );

      if($latestDelivered !== false) {
        $nextPostTitleObject = ContentEngageController::getInstance()->entityPostTitle->getById($latestDelivered->post_title_id);

        if(!empty($nextPostTitleObject)) {
          $this->displayTitleSave[$id] = $nextPostTitleObject->title;
          return $nextPostTitleObject->title;
        }
      }

      $latestAssigned = ContentEngageController::getInstance()->entityPostAssign->getNewestByPostIdAndIPAddress(
          $id, $_SERVER['REMOTE_ADDR']
      );

      if($latestAssigned !== false) {
        $nextPostTitleObject = ContentEngageController::getInstance()->entityPostTitle->getById($latestAssigned->post_title_id);

        if(!empty($nextPostTitleObject)) {
          $this->displayTitleSave[$id] = $nextPostTitleObject->title;
          return $nextPostTitleObject->title;
        }
      }

      if(isset($_COOKIE[$this->trackingCookiePrefix . $id])) {
        $nextPostTitleObject = ContentEngageController::getInstance()->entityPostTitle->getById(
            intval($_COOKIE[$this->trackingCookiePrefix . $id])
        );

        if(!empty($nextPostTitleObject)) {
          $this->displayTitleSave[$id] = $nextPostTitleObject->title;
          return $nextPostTitleObject->title;
        }
      }

      if(ContentEngageController::getInstance()->modelSettings->automaticTitleAssociation == 1) {
        $nextPostTitleObject = ContentEngageController::getInstance()->entityPostTitle->getNaturalDeliveryTitleByAssignCount(
            $id
        );

        if(!empty($nextPostTitleObject)) {
          if(ContentEngageController::getInstance()->modelSettings->automaticTitleAssociationMode == 'cookie') {
            ContentEngageController::getInstance()->footerScript .= 'document.cookie = "' . $this->trackingCookiePrefix . $id . '=' . $nextPostTitleObject->id .
                '; expires=' . date(DATE_RFC2822, time() + (3600 * 24)) . '; path=/";' . "\n";
          } else {
            $this->_registerAssignment($nextPostTitleObject->id, $id);
          }
          $this->displayTitleSave[$id] = $nextPostTitleObject->title;
          return $nextPostTitleObject->title;
        }
      }

      return $title;
    }

    return ($this->assignedTitle == '' ? $title : $this->assignedTitle);
  }

  public function _handlePostTitleTestMode($title, $id = null) {
    $randomEntry = $this->_getRandomEntry($id);

    return ($randomEntry == false ? $title : $randomEntry->title);
  }

  public function _handlePageTitle($title, $sep) {
    return ($this->assignedTitle == '' ? $title : $this->assignedTitle) . " " . $sep . " ";
  }

  private function _handleTestMode() {
    if(get_post_meta($this->postID, ContentEngageController::getInstance()->postOptionTestMode, true) == 1) {
      $postTitleObject = $this->_getRandomEntry();

      if($postTitleObject == false)
        return false;

      $this->_deliverVariationByInformationObject($postTitleObject->id, false);

      $this->deliveredVerdict = true;

      return true;
    }

    return false;
  }

  private function _getRandomEntry($postID = false) {
    $postID = $postID == false ? $this->postID : $postID;
    $postTitleObjectList = ContentEngageController::getInstance()->entityPostTitle->getAllByPostID($postID);

    if(empty($postTitleObjectList))
      return false;

    $postTitleObject = array_slice($postTitleObjectList, rand(0, count($postTitleObjectList) - 1), 1);
    $postTitleObject = array_shift($postTitleObject);

    return $postTitleObject;
  }

  private function _registerAssignment($postTitleID, $postID = false) {
    ContentEngageController::getInstance()->entityPostTitle->increaseAssignCountByID($postTitleID);
    ContentEngageController::getInstance()->entityPostAssign->insert(array(
        'post_title_id' => $postTitleID,
        'post_id'       => ($postID == false ? $this->postID : $postID),
        'ip_address'    => $_SERVER['REMOTE_ADDR']
    ));
  }

  private function _registerDisplay($postTitleID) {
    ContentEngageController::getInstance()->entityPostTitle->increaseDisplayCountByID($postTitleID);
    ContentEngageController::getInstance()->entityPostDisplay->insert(array(
        'post_title_id' => $postTitleID,
        'post_id'       => $this->postID,
        'ip_address'    => $_SERVER['REMOTE_ADDR']
    ));
  }

  private function _setPostID() {
    $this->postID = url_to_postid( $this->_getCurrentPage() );
  }

  private function _getCurrentPage() {
    return "http://".$_SERVER['SERVER_NAME'].$_SERVER['REQUEST_URI'];
  }

  /**
   * @param $token
   * @return string
   */
  private function _simpleHexEncodingWithStrRot13($token){
    return @str_rot13(bin2hex($token));
  }

  /**
   * @param $token
   * @return string
   */
  private function _simpleHexDecodingWithStrRot13($token){
    return @pack("H*",str_rot13($token));
  }

  public function verdictExists($postID = false) {
    $postID = $postID == false ? $this->postID : $postID;

    $uniqueDeliveries = ContentEngageController::getInstance()->entityPostAssign->getAllUniqueByPostID($postID);

    $splitTestMargin = get_post_meta($postID, ContentEngageController::getInstance()->postOptionSplitTestingMargin, true);
    $splitTestMargin = $splitTestMargin == '' ? ContentEngageController::getInstance()->modelSettings->defaultSplitTestingMargin : $splitTestMargin;

    return (count($uniqueDeliveries) > $splitTestMargin);
  }
}